/*** 
 * @Author: pengcheng
 * @Date: 2021-06-24 10:27:50
 * @LastEditTime: 2021-08-06 14:41:01
 * @LastEditors: pengcheng
 * @Description: 
 * @FilePath: /kylin-daq/src/DataTransmit.h
 */
#ifndef DATATRANSMIT_H
#define DATATRANSMIT_H

#include <QtCore/QtCore>
#include <QtNetwork/QtNetwork>

#include <openssl/rsa.h>
#include <openssl/pem.h>
#include <openssl/err.h>
#include <openssl/bio.h>
#include <openssl/evp.h>
#include <QTimer>
#include "BaseModule.h"
#include "KylinDaqCommon.h"
#include <QDBusInterface>
#include <QDateTime>

class TransmitModule: public BaseModule
{
    Q_OBJECT
public:
    //	enum
    enum SendInfoStatus
    {
        OK = 0,
        FormatException = -1,
        NetworkAnomaly = -2,
        ServerException = -3,
        EncryptMd5Exception = -4,
        ServerReplyException = -5,
        UrlException = -6

    };
private:
    static constexpr const char *s_messageKey = "messages";
    //	protocol key
    static constexpr const char *s_replyErrorCodeKey = "code";
    static constexpr const char *s_replyErrorMsgKey = "msg";
    static constexpr const char *s_replyResponseBodyKey = "bo";
    //	HTTP param
    static constexpr const char *s_requestTyepValue = "application/json";
    static constexpr const int32_t s_replyValidCode = 200;
    static constexpr const int32_t s_httpReplyValidCode = 200;
    //	API url
    static constexpr const char *s_uploadUrl = "collect-info-server/collect/info/submit/event/v1";
    static constexpr const char *s_reuploadUrl = "collect-info-server/collect/info/submit/event/v1";
    static constexpr const char *s_stateInfoUploadUrl = "collect-info-server/collect/info/submit/status/v1";
    static constexpr const char *s_configUrl = "collect-info-server/collect/info/config/v1";
    static constexpr const char *s_uploadEventUrl = "kylin-data-collection-service/collect-api/submit/v1";

    // conf
    static constexpr const char *s_moduleName = "TransmitModule";
    static constexpr const char *s_uploadIntervalKey = "UploadInterval";
    static constexpr const char *s_reuploadIntervalKey = "ReuploadInterval";
    static constexpr const char *s_stateInfoUploadIntervalKey = "UploadStateInfoInterval";
    static constexpr const char *s_numOfUploadedLinesKey = "NumOfUploadedLines";
    static constexpr const char *s_numOfReuploadedLinesKey = "NumOfReuploadedLines";
    static constexpr const char *s_numOfStateInfoUploadedLinesKey = "NumOfUploadedStateInfoLines";
    static constexpr const char *s_uploadStateKey = "UploadState";
    static constexpr const char *s_uploadRealmKey = "UploadRealm";
    static constexpr const char *s_uploadRequestMaxTimeKey = "UploadRequestMaxTime";
    // HTTP HEAD
    static constexpr const char *s_httpUploadTimeStamp= "X-Upload-Timestamp";
    //
    static constexpr const char *s_uploadTimeKey="uploadTimestamp";
    //
    static constexpr const int32_t s_uploadIntervalThreshold = 120;
    //	NetworkManager
    QScopedPointer<QNetworkAccessManager> m_mannager;
    //	Timer
    QTimer *uploadTimer;
    QTimer *reuploadTimer;
    QTimer *uploadStateInfoTimer;

    struct idList{
        QList<qlonglong> successId;
        QList<qlonglong> failId;
        QList<QString> failInfo;
    };


    // singleton
    static QScopedPointer<TransmitModule> ms_instance;
    SendInfoStatus sendInfo(const QList<QString> &info, const QString &urlValue, const QString &requestHead, const int32_t timeCountMsec);
    idList sendEventInfo(const QList<qlonglong> &id, const QList<QString> &info, const QString &urlValue, const QString &requestHead, const int32_t timeCountMsec);
    SendInfoStatus getInfo(const QString &urlValue, const QString &requestHead, const QString& requestBody,const int32_t timeCountMsec,QString &reponseBody);
    // 检测url
    SendInfoStatus checkUrl(const QUrl &urlValue);
    // 整合发送信息
    SendInfoStatus mergeInfo(const QList<QString> &info,QByteArray &requestData);
    // 设置请求
    SendInfoStatus setRequest(QNetworkRequest &request,const QUrl &url,const QJsonObject &requestHeadObj);
    // 发送请求
    SendInfoStatus sendRequest(QNetworkRequest &request,const QByteArray &requestData,const int32_t timeCountMsec,int32_t &requestErrorCode,QString &requestErrorRelyStr,QString &reponseBody);
    // 打印发送数据日志
    void printSendInfoLog(const QList<QString> &info,const SendInfoStatus &requestState,int32_t requestErrorCode,const QString &requestErrorRelyStr);
    //
    void sendUploadInfoNum();
    void sendReuploadInfoNum();
    void sendStateInfoNum();
    // 判断网络是否连接
    bool networkIsConnected();
    // 组合状态消息

    QString createStateInfo(const QString &packageInfo, const QString &uploadedMessage, const QString &tids, const QString &createTimeStamp,const KylinDaqCommon::UploadStateEnum &uploadState);
public:
    //	structure function
    explicit TransmitModule(QObject *parent = nullptr);
    virtual ~TransmitModule();
    //	singleton function
    static TransmitModule *instance(void);
public slots:
    void updateProperty(const QJsonObject &conf, const QJsonObject &defaultConf);
    void uploadInfo(const QList<qlonglong> &id,const QList<QString> &info,const QList<QString> & status, const QString &requestHead);
    void reuploadInfo(const QList<qlonglong> &id,const QList<QString> &info,const QList<QString> & status, const QString &requestHead);

    void uploadEventInfo(const QList<qlonglong> &id,const QList<QString> &info,const QList<QString> & status, const QString &requestHead);
    void reuploadEventInfo(const QList<qlonglong> &id,const QList<QString> &info,const QList<QString> & status, const QString &requestHead);
    void uploadStateInfo(const QList<QString> &packageInfoList,const QList<QString> &uploadedMessageList, const QList<QString> &tidsList, const QList<QString> &createTimeStampList,const QList<KylinDaqCommon::UploadStateEnum> & uploadStateList, const QString &requestHead);
    void setUploadTimerPollingInterval(qlonglong sec);
    void setReuploadTimerPollingInterval(qlonglong sec);
    void setUploadStateInfoTimerPollingInterval(qlonglong sec);
    void printConf();
    void getConfig();
signals:
    void uploadInfoSignal(qlonglong num);
    void uploadEventInfoSignal(qlonglong num);
    void reuploadInfoSignal(qlonglong num);
    void reuploadEventInfoSignal(qlonglong num);
    void uploadStateInfoSignal(qlonglong num);
    void retUploadState(const QList<qlonglong> &id,const QList<QString> &info,const QList<QString> & status, const TransmitModule::SendInfoStatus &sendInfoStatus);
    void retEventUploadState(const QList<qlonglong> &id,const QList<QString> &info,const QList<QString> & status, const TransmitModule::SendInfoStatus &sendInfoStatus);
    void retReuploadState(const QList<qlonglong> &id,const QList<QString> &info,const QList<QString> & status, const TransmitModule::SendInfoStatus &sendInfoStatus);
    void retEventReuploadState(const QList<qlonglong> &id,const QList<QString> &info,const QList<QString> & status, const TransmitModule::SendInfoStatus &sendInfoStatus);
    void retUploadStateInfoState(const QList<QString> &packageInfoList, const TransmitModule::SendInfoStatus &sendInfoStatus);
    void sendConfig(const QString &config);

    //	property
private:
    //	Q_PROPERTY
    Q_PROPERTY(qlonglong uploadInterval READ uploadInterval WRITE setUploadInterval NOTIFY uploadIntervalChanged)
    Q_PROPERTY(qlonglong reuploadInterval READ reuploadInterval WRITE setReuploadInterval NOTIFY reuploadIntervalChanged)
    Q_PROPERTY(qlonglong uploadStateInfoInterval READ uploadStateInfoInterval WRITE setUploadStateInfoInterval NOTIFY uploadStateInfoIntervalChanged)
    Q_PROPERTY(qlonglong numOfUploadedLines READ numOfUploadedLines WRITE setNumOfUploadedLines NOTIFY numOfUploadedLinesChanged)
    Q_PROPERTY(qlonglong numOfReuploadedLines READ numOfReuploadedLines WRITE setNumOfReuploadedLines NOTIFY numOfReuploadedLinesChanged)
    Q_PROPERTY(qlonglong numOfUploadedStateInfoLines READ numOfUploadedStateInfoLines WRITE setNumOfUploadedStateInfoLines NOTIFY numOfUploadedStateInfoLinesChanged)
    Q_PROPERTY(QList<QString> uploadRealm READ uploadRealm WRITE setUploadRealm NOTIFY uploadRealmChanged)
    Q_PROPERTY(KylinDaqCommon::UploadState uploadState READ uploadState WRITE setUploadState NOTIFY uploadStateChanged)
    Q_PROPERTY(qlonglong uploadRequestMaxTime READ uploadRequestMaxTime WRITE setUploadRequestMaxTime NOTIFY uploadRequestMaxTimeChanged)
    //	property var
    qlonglong m_numOfUploadedLines;
    qlonglong m_numOfReuploadedLines;
    qlonglong m_uploadInterval;
    qlonglong m_reuploadInterval;
    KylinDaqCommon::UploadState m_uploadState;
    QList<QString> m_uploadRealm;
    qlonglong m_uploadRequestMaxTime;

    qlonglong m_numOfUploadedStateInfoLines;

    qlonglong m_uploadStateInfoInterval;

public:
    //	property function
    qlonglong numOfUploadedLines() const
    {
        return m_numOfUploadedLines;
    }

    qlonglong numOfReuploadedLines() const
    {
        return m_numOfReuploadedLines;
    }

    qlonglong uploadInterval() const
    {
        return m_uploadInterval;
    }

    qlonglong reuploadInterval() const
    {
        return m_reuploadInterval;
    }

    KylinDaqCommon::UploadState uploadState() const
    {
        return m_uploadState;
    }
    QList<QString> uploadRealm() const
    {
        return m_uploadRealm;
    }

    qlonglong uploadRequestMaxTime() const
    {
        return m_uploadRequestMaxTime;
    }

    qlonglong numOfUploadedStateInfoLines() const
    {
        return m_numOfUploadedStateInfoLines;
    }

    qlonglong uploadStateInfoInterval() const
    {
        return m_uploadStateInfoInterval;
    }

public slots:
    //	property slots
    void setNumOfUploadedLines(qlonglong numOfUploadedLines)
    {
        if (m_numOfUploadedLines == numOfUploadedLines)
            return;

        m_numOfUploadedLines = numOfUploadedLines;
        emit numOfUploadedLinesChanged(m_numOfUploadedLines);
    }

    void setNumOfReuploadedLines(qlonglong numOfReuploadedLines)
    {
        if (m_numOfReuploadedLines == numOfReuploadedLines)
            return;

        m_numOfReuploadedLines = numOfReuploadedLines;
        emit numOfReuploadedLinesChanged(m_numOfReuploadedLines);
    }

    void setUploadInterval(qlonglong uploadInterval)
    {
        if (m_uploadInterval == uploadInterval)
            return;

        m_uploadInterval = uploadInterval;
        emit uploadIntervalChanged(m_uploadInterval);
    }

    void setReuploadInterval(qlonglong reuploadInterval)
    {
        if (m_reuploadInterval == reuploadInterval)
            return;

        m_reuploadInterval = reuploadInterval;
        emit reuploadIntervalChanged(m_reuploadInterval);
    }

    void setUploadState(KylinDaqCommon::UploadState uploadState)
    {
        if (m_uploadState == uploadState)
            return;

        m_uploadState = uploadState;
        emit uploadStateChanged(m_uploadState);
    }

    void setUploadRealm(QList<QString> uploadRealm)
    {
        if (m_uploadRealm == uploadRealm)
            return;

        m_uploadRealm = uploadRealm;
        emit uploadRealmChanged(m_uploadRealm);
    }

    void setUploadRequestMaxTime(qlonglong uploadRequestMaxTime)
    {
        if (m_uploadRequestMaxTime == uploadRequestMaxTime)
            return;

        m_uploadRequestMaxTime = uploadRequestMaxTime;
        emit uploadRequestMaxTimeChanged(m_uploadRequestMaxTime);
    }

    void setNumOfUploadedStateInfoLines(qlonglong numOfStateInfoUploadedLines)
    {
        if (m_numOfUploadedStateInfoLines == numOfStateInfoUploadedLines)
            return;

        m_numOfUploadedStateInfoLines = numOfStateInfoUploadedLines;
        emit numOfUploadedStateInfoLinesChanged(m_numOfUploadedStateInfoLines);
    }

    void setUploadStateInfoInterval(qlonglong uploadStateInfoInterval)
    {
        if (m_uploadStateInfoInterval == uploadStateInfoInterval)
            return;

        m_uploadStateInfoInterval = uploadStateInfoInterval;
        emit uploadStateInfoIntervalChanged(m_uploadStateInfoInterval);
    }

signals:
    //	property signals
    void numOfUploadedLinesChanged(qlonglong numOfUploadedLines);
    void numOfReuploadedLinesChanged(qlonglong numOfReuploadedLines);
    void uploadIntervalChanged(qlonglong uploadInterval);
    void reuploadIntervalChanged(qlonglong reuploadInterval);
    void uploadStateChanged(KylinDaqCommon::UploadState uploadState);
    void uploadRealmChanged(QList<QString> uploadRealm);
    void uploadRequestMaxTimeChanged(qlonglong uploadRequestMaxTime);
    void numOfUploadedStateInfoLinesChanged(qlonglong numOfUploadedStateInfoLines);
    void uploadStateInfoIntervalChanged(qlonglong uploadStateInfoInterval);
};

#endif
