#include <dlfcn.h>
#include <execinfo.h>
#include <signal.h>
#include <unistd.h>
#include <libkylog.h>
#include "LogControl.h"
#include "SingleApplication.h"
#include "DataCommon.h"
#include "BasicInfoModule.h"


void trappingSignals(void);
static void crashHandler(int sig);


static void logOutput(QtMsgType type, const QMessageLogContext &context, const QString &msg)
{
    QByteArray localMsg = msg.toLocal8Bit();
    const char *message = localMsg.constData();
    const char *file = context.file ? context.file : "";
    const char *function = context.function ? context.function : "";

    switch (type) {
    case QtDebugMsg:
        klog_debug("%s (%s:%u,%s)\n", message, file, context.line, function);
        break;
    case QtInfoMsg:
        klog_info("%s (%s:%u,%s)\n", message, file, context.line, function);
        break;
    case QtWarningMsg:
        klog_warning("%s (%s:%u,%s)\n", message, file, context.line, function);
        break;
    case QtCriticalMsg:
        klog_err("%s (%s:%u,%s)\n", message, file, context.line, function);
        break;
    case QtFatalMsg:
        klog_emerg("%s (%s:%u,%s)\n", message, file, context.line, function);
        break;
    }

    return;
}

int main(int argc, char *argv[])
{
    qInstallMessageHandler(logOutput);
    //  设置环境变量
    setenv("LANGUAGE","en",1);
    //  设置日志信息
    //  设置退出信号函数
    // trappingSignals();
    SingleApplication application(argc, argv);
    if(!application.isRunningFunction())
    {
        DataCommon dataCommon;
        return application.exec();
    }

    sleep(1);
    return 0;
}


static void crashHandler(int sig)
{
    int32_t buffSize = 1024;
    signal(sig, SIG_IGN);
    size_t size = 0;
    char **strings = NULL;
    size_t i = 0, j = 0;
    void *array[20];
    size = backtrace(array, 20);
    strings = const_cast<char **> (backtrace_symbols(array, size));
    qCritical().noquote() << QString("!!!--- %1 received signal: %2=%3 ---!!!")
        .arg(QDateTime::currentDateTime().toString(
            "yyyy-MM-dd hh:mm:ss.zzz"))
        .arg(sig)
        .arg(strsignal(sig))
        .toUtf8();
    QString trackLog;
    for (i = 0; i < size; i++) {
        trackLog.append(QString("%1 %2\n").arg(i).arg(strings[i]));
    }
    qCritical().noquote() << trackLog.toUtf8();
    free(strings);
    exit(128 + sig);
}

void trappingSignals(void)
{
    if (signal(SIGCHLD, SIG_IGN) == SIG_ERR) //忽略子进程已经停止或退出
    {
        //注册SIGCHLD信号失败
        perror("signal error");
    }
    if (signal(SIGSEGV, crashHandler) == SIG_ERR) //无效内存段访问
    {
        //注册SIGSEGV信号失败
        perror("signal error");
    }
    if (signal(SIGILL, crashHandler) == SIG_ERR) //非法指令
    {
        //注册SIGILL信号失败
        perror("signal error");
    }
    if (signal(SIGTERM, crashHandler) == SIG_ERR) //终止
    {
        //注册SIGTERM信号失败
        perror("signal error");
    }
    if (signal(SIGHUP, crashHandler) == SIG_ERR) //系统挂断
    {
        //注册SIGHUP信号失败
        perror("signal error");
    }
    if (signal(SIGABRT, crashHandler) == SIG_ERR) //进程停止运行
    {
        //注册SIGABRT信号失败
        perror("signal error");
    }
}
